/*
  Modified Baudin and Smith's method for complex divide with enhancements
  Test ratio for less than or equal to RMIN
  Test c&d < RMIN2, scale to reduce underflows
 */

#include <stdio.h>
#include <math.h>
#include <float.h>
#include <complex.h>

#define RMIN DBL_MIN
#define RMIN2 DBL_EPSILON
#define RMINSCAL 1/DBL_EPSILON


double complex cdivd(double complex x, double complex y)
{
  double a, b, c, d, ratio, denom, xx, yy;
  double ax, bx, cx, dx;
  double complex res;

  a = creal(x);
  b = cimag(x);
  c = creal(y);
  d = cimag(y);
  if (fabs(c) < fabs(d))
    {
      if (fabs(d) < RMIN2)
	{
	  a = a * RMINSCAL;
	  b = b * RMINSCAL;
	  c = c * RMINSCAL;
	  d = d * RMINSCAL;
	}
      ratio = c / d;
      denom = (c * ratio) + d;
      if ( fabs(ratio) > RMIN)
	{
	  xx = ((a * ratio) + b) / denom;
	  yy = ((b * ratio) - a) / denom;
	}
      else
	{
	  xx = (c * (a / d) + b) / denom;
	  yy = (c * (b / d) - a) / denom;
	}
    } 
  else
    {
      if (fabs(c) < RMIN2)
	{
	  a = a * RMINSCAL;
	  b = b * RMINSCAL;
	  c = c * RMINSCAL;
	  d = d * RMINSCAL;
	}
      ratio = d / c;
      denom = (d * ratio) + c;
      if ( fabs(ratio) > RMIN)
	{
	  xx = ((b * ratio) + a) / denom;
	  yy = (b - (a * ratio)) / denom;
	}
      else
	{
	  xx = (a + (d * (b / c))) / denom;
	  yy = (b - (d * (a /c))) / denom;
	}
    }

  /* Recover infinities and zeros that computed as NaN+iNaN; the only cases
     are nonzero/zero, infinite/finite, and finite/infinite.  */
  if (isnan (xx) && isnan (yy))
    {
      ax = creal(x);
      bx = cimag(x);
      cx = creal(y);
      dx = cimag(y);
      denom = (c + d * ratio);
      if (denom == 0.0 && (!isnan (ax) || !isnan (bx)))
	{
	  xx = copysignl (INFINITY, cx) * ax;
	  yy = copysignl (INFINITY, cx) * bx;
	}
      else if ((isinf (ax) || isinf (bx))
	       && isfinite (cx) && isfinite (dx))
	{
	  ax = copysignl (isinf (ax) ? 1 : 0, a);
	  bx = copysignl (isinf (bx) ? 1 : 0, b);
	  xx = INFINITY * (ax * cx + bx * dx);
	  yy = INFINITY * (bx * cx - ax * dx);
	}
      else if ((isinf (c) || isinf (d))
	       && isfinite (a) && isfinite (b))
	{
	  cx = copysignl (isinf (cx) ? 1 : 0, c);
	  dx = copysignl (isinf (dx) ? 1 : 0, d);
	  xx = 0.0 * (ax * cx + bx * dx);
	  yy = 0.0 * (bx * cx - ax * dx);
	}
      else
	{
	  xx = 0.0/0.0;
	  yy = 0.0/0.0;
	}
    }
  res = (double complex)(xx + yy * I);
  return res;
}
