/*
  Smith's method for complex divide 
  as implemented in libgcc on Aug 24, 2020
 */

#include <stdio.h>
#include <math.h>
#include <complex.h>

float complex cdivf(float complex x, float complex y)
{
  double a, b, c, d, ratio, denom;
  float xx, yy;
  float ax, bx, cx, dx;
  float complex res;

  a = crealf(x);
  b = cimagf(x);
  c = crealf(y);
  d = cimagf(y);
  denom = (c * c) + (d * d);
  xx = ((a * c) + (b * d)) / denom;
  yy = ((b * c) - (a * d)) / denom;

  /* Recover infinities and zeros that computed as NaN+iNaN; the only cases
     are nonzero/zero, infinite/finite, and finite/infinite.  */
  if (isnan (xx) && isnan (yy))
    {
      ax = creal(x);
      bx = cimag(x);
      cx = creal(y);
      dx = cimag(y);
      denom = (c + d * ratio);
      if (denom == 0.0 && (!isnan (ax) || !isnan (bx)))
	{
	  xx = copysignl (INFINITY, cx) * ax;
	  yy = copysignl (INFINITY, cx) * bx;
	}
      else if ((isinf (ax) || isinf (bx))
	       && isfinite (cx) && isfinite (dx))
	{
	  ax = copysignl (isinf (ax) ? 1 : 0, a);
	  bx = copysignl (isinf (bx) ? 1 : 0, b);
	  xx = INFINITY * (ax * cx + bx * dx);
	  yy = INFINITY * (bx * cx - ax * dx);
	}
      else if ((isinf (c) || isinf (d))
	       && isfinite (a) && isfinite (b))
	{
	  cx = copysignl (isinf (cx) ? 1 : 0, c);
	  dx = copysignl (isinf (dx) ? 1 : 0, d);
	  xx = 0.0 * (ax * cx + bx * dx);
	  yy = 0.0 * (bx * cx - ax * dx);
	}
      else
	{
	  xx = 0.0/0.0;
	  yy = 0.0/0.0;
	}
    }
  res = (float complex)(xx + yy * I);
  return res;
}
