/* function to generate a random double precision number
   with uniformly distributed exponent and uniformly distributed mantissa
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* includes for debugging printouts */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* argument x is to fool compiler into not optimizing out
   repeated calls to mkran */
double mkrand(double x)
{
  union
  {
    long long int i;	/* must be 64-bit integer type */
    double d;		/* must be 64-bit IEEE 754 type */
  } value;
  long long int exp,m,m1,m2,m3,sign;
  long long tiny, shift;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
   */
  do {
    /*
      Make 1 out of 40 cases have a subnormal exponent
      to test the full range of exponents (+1023 to -1076)
      instead of simply +1023 to -1024.
     */
    tiny = ((long long int)(random() >> 8) & 0x7ff)/51;
    if (tiny == 1) {
      exp = 0;
      shift = ((long long int)(random() >> 8) & 0x7ff)/40;
      if (shift >= 50) shift = 0;
      if (shift >= 34) {
	m1 = 0;
	m2 = 0;
	m3 = (((random() >> 8)| 0x8000) & 0xffff) >> shift-34;
      } else if (shift >= 18) {
	m1 = 0;
	m2 = (((random() >> 8) | 0x8000) & 0xffff) >> shift-18;
	m3 = ((random() >> 8) & 0xffff);
      } else {
	m1 = (((random() >> 4) | 0x80000) & 0xfffff) >> shift;
	m2 = ((random() >> 8) & 0xffff);
	m3 = ((random() >> 8) & 0xffff);
      }
    } else {
      exp = ((long long int)(random() >> 12) & 0x7ff) << 52;
      m1 = ((random() >> 4) & 0xfffff);
      m2 = ((random() >> 8) & 0xffff);
      m3 = ((random() >> 8) & 0xffff);
    }
    value.i = exp | (m1 << 32) | (m2 << 16) | m3;
    sign = (random() >> 8);
    if (sign & 1) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
