/* function to generate a random double precision number
   with uniformly distributed exponent and uniformly distributed mantissa
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* includes for debugging printouts */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* argument x is to fool compiler into not optimizing out
   repeated calls to mkran */
double mkrand(double x)
{
  union
  {
    long long int i;	/* must be 64-bit integer type */
    double d;		/* must be 64-bit IEEE 754 type */
  } value;
  long long int exp,m,m1,m2,m3,sign;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
  */
  do {
    /* set exponent between -512 and 511*/
    exp = ((long long int)(random() >> 12) & 0x7ff);
    if (exp > 0x600) {
      exp = exp & 0x5ff;
    } else if (exp < 0x200){
      exp = exp | 0x200;
    }
    exp = exp << 52;

    m1 = ((random() >> 4) & 0xfffff);
    m2 = ((random() >> 8) & 0xffff);
    m3 = ((random() >> 8) & 0xffff);
    value.i = exp | (m1 << 32) | (m2 << 16) | m3;
    sign = (random() >> 8);
    if (sign & 1) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
