/* function to generate a random float precision number
   with uniformly distributed exponent and uniformly distributed mantissa
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* for debugging printouts */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* argument x is to fool compiler into not optimizing out
   repeated calls to mkran */
float mkranf(float x)
{
  union
  {
    long i;		/* must be 32-bit integer type */
    float d;		/* must be 32-bit IEEE 754 type */
  } value;
  long exp,m,m1,m2,m3,sign;
  long tiny, shift;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
  */
  do {
    /*
      Make 1 out of 20 cases have a subnormal exponent
      to test the full range of exponents (+1023 to -1076)
      instead of simply +127 to -126.
     */
    tiny = ((long)(random() >> 8) & 0xff)/12;
    if (tiny == 1) {
      exp = 0;
      shift = ((long)(random() >> 8) & 0xfff)/178;
      /* shift is >= 0 and < 23 */
      if (shift >= 23) shift = 0;
      if (shift >= 12) {
	m1 = 0;
	m2 = (((random() >> 8) | 0x800) & 0xfff) >> shift-12;
      } else {
	m1 = ((random() >> 8) & 0x7ff) >> shift;
	m2 = ((random() >> 8) & 0xfff);
      }
    } else {
      exp = ((long)(random() >> 12) & 0xff) << 23;
      m1 = ((random() >> 8) & 0x7ff);
      m2 = ((random() >> 8) & 0xfff);
    }
    value.i = exp | (m1 << 12) | m2;
    sign = (random() >> 8);
    if (sign & 1) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
