/* function to generate a random float precision number
   with uniformly distributed exponent and uniformly distributed mantissa
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* for debugging printouts */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* argument x is to fool compiler into not optimizing out
   repeated calls to mkranf */
float mkranf(float x)
{
  union
  {
    long i;	/* must be 64-bit integer type */
    float d;		/* must be 64-bit IEEE 754 type */
  } value;
  long exp, m1, m2, sign;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
  */
  do {
    /* set exponent between -63 and 63*/
    exp = ((long long int)(random() >> 12) & 0xff);
    if (exp > 0xbe) {
      exp = exp & 0xbe;
    } else if (exp < 0x40){
      exp = exp | 0x40;
    }
    exp = exp << 23;

    m1 = ((random() >> 8) & 0x7ff);
    m2 = ((random() >> 8) & 0xfff);
    value.i = exp | (m1 << 12) | m2;
    sign = (random() >> 8);
    if (sign & 1) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
