/* function to generate a random double precision number
   with uniformly distributed exponent and uniformly distributed mantissa
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* for debugging printouts */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* argument x is to fool compiler into not optimizing out
   repeated calls to mkranf */
_Float16 mkranh(_Float16 x)
{
  union
  {
    int16_t i;		/* must be 16-bit integer type */
    _Float16 d;		/* must be 16-bit IEEE 754 type */
  } value;
  long exp, m1, sign;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
  */
  do {
    /* set exponent between -7 and 17*/
    exp = ((long int)(random() >> 12) & 0x78);
    if (exp > 0x3f) {
      exp = exp & 0x3f;
    } else if (exp < 0x20){
      exp = exp | 0x20;
    }
    exp = exp << 8;

    m1 = ((random() >> 12) & 0x3ff);
    value.i = exp | m1;
    sign = (random() >> 8);
    if (sign & 1) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
