/* Function to generate a random long double (128-bit format)
   with uniformly distributed exponent and uniformly distributed mantissa.
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* for debugging printouts  */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* Argument x is to fool compiler into not optimizing out
   repeated calls to mkranx.  */
long double mkranx(long double x)
{
  union
  {
    long i[2];			/* Must be 2x64-bit integer type.  */
    long double d;		/* Must be 128-bit IEEE 754 type.  */
  } value;
  long exp, m1, m2, m3, m4, m5, m6, m7, sign;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
  */
  do {
    /* Set exponent between -16*1024 and 16*1024.  */
    exp = ((random() >> 12) & 0x7fff);
    /* Generate 112 bit mantissa.  */
    m1 = ((random() >> 8) & 0xffff);
    m2 = ((random() >> 8) & 0xffff);
    m3 = ((random() >> 8) & 0xffff);
    m4 = ((random() >> 8) & 0xffff);
    m5 = ((random() >> 8) & 0xffff);
    m6 = ((random() >> 8) & 0xffff);
    m7 = ((random() >> 8) & 0xffff);
    value.i[0] = (m4 << 48) | (m5 << 32) | (m6 << 16) | m7;
    value.i[1] = exp | (m1 << 32) | (m2 << 16) | m3;
    sign = (random() >> 8)&1;
    if (sign) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
