/* Function to generate a random long double (80 bit Intel format)
   with uniformly distributed exponent and uniformly distributed mantissa.
   All numbers will not have the maximum or minimum exponent bit set.
   Actual distribution will be heavily skewed away from numbers near 1.0.
   One half of numbers will be positive and one half will be negative.
*/

/* for debugging printouts */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* Argument x is to fool compiler into not optimizing out
   repeated calls to mkranx.  */
long double mkranx(long double x)
{
  union
  {
    long i[2];			/* Must be 2x64-bit integer type.  */
    long double d;		/* Must be 128-bit IEEE 754 type.  */
  } value;
  long exp, m1, m2, m3, m4, sign;
  /* We avoid using low bits of values returned from random
     due to some random number generators having non-random
     in their low bits. The glibc random is not known to have
     such defects, but we avoid those bits anyway.
  */
  do {
    /* Set exponent between -8*1024 and 8*1024.  */
    exp = ((random() >> 12) & 0x7fff);
    if (exp > 0x6000) {
      exp = exp & 0x5fff;
    } else if (exp < 0x2000){
      exp = exp | 0x2000;
    }
    /* Generate 63 bit mantissa.  */
    m1 = ((random() >> 8) & 0x7fff);
    m2 = ((random() >> 8) & 0xffff);
    m3 = ((random() >> 8) & 0xffff);
    m4 = ((random() >> 8) & 0xffff);
    /* Insure bit 64 is zero.  */
    m1 = m1 | 0x8000;
    value.i[0] = (m1 << 48) | (m2 << 32) | (m3 << 16) | m4;
    value.i[1] = exp;
    sign = (random() >> 8)&1;
    if (sign) { value.d = value.d * -1.0;}
  } while (isnan(value.d));

  return value.d;
}
