/*
  perf_cdivx.c
  Performance test for complex divide.
  Put complex divide in file cdivx_xx.c.
  Compile test with
  cc cdivx_xx.c perf_cdivx.c mkranx.c -O2 -lrt -o perf_cdivx
  ./perf_cdivx
  Time is nanoseconds per complex divide.
  Zeros is number of times results shows zero.
 */
#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include <time.h>

extern long double mkranx(long double);
__complex long double cdivx(__complex long double,__complex long double);

/* Number of values to test.  */
#define PERF_CNT 100000
/* Number of runs for computing median.  */
#define MED_CNT 9

__complex long double perf_x[PERF_CNT];
__complex long double perf_y[PERF_CNT];
__complex long double sum[PERF_CNT];

#define NSECS 1000000000
#define TIMEIT(s) clock_gettime(CLOCK_REALTIME,s)

long double infp, infm;

/* Return true if x/y can be computed in long double precision.  */
int zzvalid(__complex long double a)
{
  /*  printf("valid = %-24.13a %-24.13ai\n",__real__ a,__imag__ a);*/
  return ((__real__ a != infp) && (__real__ a != infm) &&
          (__imag__ a != infp) && (__imag__ a != infm) );
}

void init_vals(int k)
{
  int i;
  long double z;
  __complex long double x,y,qq;
  /* Initialize k random values for complex divide tests.  */
  z = 1.0;
  i = 0;
  while (i < k) 
    {
      __real__ x = mkranx(z);
      __imag__ x = mkranx(z);
      __real__ y = mkranx(z);
      __imag__ y = mkranx(z);
      qq = x/y;
      if (zzvalid(qq))
	{
	  perf_x[i] = x;
	  perf_y[i] = y;
	  sum[i] = x;
	  i++;
	}
    }
}


long deltat(struct timespec s, struct timespec e) {
  long sec,nsec;
  sec = e.tv_sec - s.tv_sec;
  nsec = e.tv_nsec - s.tv_nsec;
  return ( sec*NSECS + nsec );
}

/* Compute median by simple bubble sort followed by selecting mid-point.  */
long get_median(long vals[],int cnt)
{
  int i,j;
  long tmp;
  for (j = cnt; j!=0; j--)
    {
      for (i = 0; i < j-2 ; i++)
	{
	  if (vals[i] > vals[i+1])
	    {
	      tmp = vals[i];
	      vals[i] = vals[i+1];
	      vals[i+1] = tmp;
	    }
	}
    }
  return vals[(cnt-1)/2];
}

int main() {
  long xxx_nsec[MED_CNT];
  long xx_nsec;
  double nsec1;
  int i,k,cnt;
  struct timespec tstart, tend;

  infp = 1.0/0.0;
  infm = -1.0/0.0;

  cnt = 0;
  
  /*
    Compute complex divide for PERF_CNT values.
    Measure MED_CNT times and take median to minimize system noise effects.
   */
  init_vals(PERF_CNT);

  for (k = 0; k < MED_CNT; k++)
    {
      TIMEIT(&tstart);
      for (i = 0; i < PERF_CNT; i++) 
	{
	  sum[i] = cdivx(perf_x[i],perf_y[i]);
	}
      TIMEIT(&tend);
      xxx_nsec[k] = deltat(tstart,tend);
    }

  xx_nsec = get_median(xxx_nsec,MED_CNT);

  /* Print results.  */
  nsec1 = (double)xx_nsec/(double)(PERF_CNT);
  printf("   %04.1f nsecs/complex divide",nsec1);

  /* To limit optimization removing timed tests.  */
  for (i = 0; i < PERF_CNT; i++)
    {
      if ((__real__ sum[i] == 0.0) && (__imag__ sum[i] == 0.0))
	{
	  cnt++;
	}
    }
  printf("  Zeros: %9d\n", cnt);

}
